﻿using UnityEngine;
using System.Collections;

public class CustomTileSetControl : OnlineMapsTileSetControl
{
	private Vector3 m_originalPosition;
	private Quaternion m_originalRotation;
	private Vector3 m_originalScale;
	private Vector3 m_mapCenter;
	private float m_nextScaleModifier = 1;

	public void Awake ()
	{
		m_originalPosition = transform.localPosition;
		m_originalRotation = transform.localRotation;
		m_originalScale = transform.localScale;
	}

	protected override void UpdateGestureZoom ()
	{
		if (!smoothZoom)
        {
			base.UpdateGestureZoom ();
			return;
		}
		
		if (!allowUserControl) return;
		
		if (Input.touchCount == 2)
        {
			isMapDrag = false;
			smoothZoomStarted = true;

			Vector2 p1 = Input.GetTouch(0).position;
			Vector2 p2 = Input.GetTouch(1).position;
			float distance = (p1 - p2).magnitude;
			
			if (lastGestureDistance != 0)
            {
				Vector2 center = Vector2.Lerp (p1, p2, 0.5f);

				RaycastHit hit;
				if (!cl.Raycast (activeCamera.ScreenPointToRay (center), out hit, OnlineMapsUtils.maxRaycastDistance)) return;

				float scale = 1;

                double px, py;
                GetCoords(out px, out py, center);

                if (!invertTouchZoom) scale = distance / lastGestureDistance;
				else scale = lastGestureDistance / distance;

				float initialScale = transform.localScale.x;
				transform.localScale *= scale * m_nextScaleModifier;

				float floatOffset = Mathf.Log (transform.localScale.x, 2);
				int offset = floatOffset > 0 ? Mathf.FloorToInt (floatOffset) : Mathf.CeilToInt (floatOffset);
				m_nextScaleModifier = 1;

                bool zoomChanged = false;

                if (offset != 0)
                {
					ZoomOnPoint (offset, center);
                    m_nextScaleModifier = Mathf.Pow (2, -offset);
                    zoomChanged = true;
                }
				
				if (transform.localScale.x < 0.9)
                {
					ZoomOnPoint (offset - 1, center);
					m_nextScaleModifier = 2;
                    zoomChanged = true;
                }

				float scaleDifference = transform.localScale.x / initialScale - 1;
				Vector3 smoothZoomOffset = transform.position - hit.point;
				transform.position += new Vector3 (smoothZoomOffset.x, 0, smoothZoomOffset.z) * scaleDifference;

                if (zoomChanged)
                {
                    double npx, npy;
                    GetCoords(out npx, out npy, center);
                    api.projection.CoordinatesToTile(px, py, api.zoom, out px, out py);
                    api.projection.CoordinatesToTile(npx, npy, api.zoom, out npx, out npy);
                    double cpx, cpy;
                    api.GetPosition(out cpx, out cpy);
                    api.projection.CoordinatesToTile(cpx, cpy, api.zoom, out cpx, out cpy);
                    cpx -= npx - px;
                    cpy -= npy - py;
                    api.projection.TileToCoordinates(cpx, cpy, api.zoom, out cpx, out cpy);
                    api.SetPosition(cpx, cpy);
                }

				ComputePinSizes (transform.localScale.x);
			}
			lastGestureDistance = distance;
		}
        else
        {
			if (smoothZoomStarted)
            {
				lastGestureDistance = 0;
				Vector2 center = new Vector2 (Screen.width / 2, Screen.height / 2);
				double ox, oy;
				if (GetCoords (out ox, out oy, center))
                {
					api.SetPosition (ox, oy);
					float scaleDifference = m_originalScale.x - transform.localScale.x;
					transform.localPosition = m_originalPosition + new Vector3 (api.tilesetSize.x / -2, 0, api.tilesetSize.y / 2) * scaleDifference;
				}
				ComputePinSizes (transform.localScale.x);
			}
			smoothZoomStarted = false;
		}
	}

	public void ResetZoom ()
	{
		transform.localPosition = m_originalPosition;
		transform.localRotation = m_originalRotation;
		transform.localScale = m_originalScale;
	}

	private void ComputePinSizes (float scale)
	{
		/*float inverseScale = 1 / (scale + 1);
		for (int i = 0; i < OnlineMaps.instance.markers.Length; i++) {
			OnlineMaps.instance.markers [i].scale *= inverseScale;
		}*/
	}
}
